package order

import (
	"context"
	"git.downtown8.com/rms/g-common/g-common/constants/errorcode"
	"git.downtown8.com/rms/g-orders/hera/kitex_gen/orderconfig"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/kitex_gen/hqOrderReceipt"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/kitex_gen/middleplatform/printer"
	"github.com/jinzhu/copier"
)

func (o *OrderService) CreateOrderReceipt(ctx context.Context, req *hqOrderReceipt.CreateOrderReceiptReq) (res *hqOrderReceipt.CreateOrderReceiptResp, err error) {
	var receiptList []*orderconfig.Receipt
	for _, v := range req.GetReceiptList() {
		receiptList = append(receiptList, &orderconfig.Receipt{
			Receipt: v.GetReceipt(),
			EatType: v.GetEatType(),
			Channel: v.GetChannel(),
			Count:   v.GetCount(),
			Scene:   v.GetScene(),
		})
	}

	r, err := o.rpcClient.OrderCenter.CreateOrderReceipt(ctx, &orderconfig.CreateOrderReceiptReq{
		Payload: &orderconfig.OrderReceipt{
			BrandId:            req.GetBrandId(),
			StoreId:            req.GetStoreId(),
			Name:               req.GetName(),
			EnableGraphicPrint: req.GetEnableGraphicPrint(),
			EnableBeep:         req.GetEnableBeep(),
			ReceiptList:        receiptList,
			StoreTableAreaNo:   req.GetStoreTableAreaNo(),
			StoreTableAll:      req.GetStoreTableAll(),
		},
	})
	if err != nil {
		return &hqOrderReceipt.CreateOrderReceiptResp{
			Code: errorcode.ERR_CODE_NOT_SET,
			Msg:  err.Error(),
		}, nil
	}
	if r.GetCode() != 0 {
		return &hqOrderReceipt.CreateOrderReceiptResp{
			Code: r.GetCode(),
			Msg:  r.GetMsg(),
		}, nil
	}
	data := new(hqOrderReceipt.CreateOrderReceiptRespData)
	copier.Copy(data, r.GetData())
	return &hqOrderReceipt.CreateOrderReceiptResp{
		Code: r.GetCode(),
		Msg:  r.GetMsg(),
		Data: data,
	}, nil
}

func (o *OrderService) GetOrderReceiptList(ctx context.Context, req *hqOrderReceipt.GetOrderReceiptListReq) (res *hqOrderReceipt.GetOrderReceiptListResp, err error) {
	r, err := o.rpcClient.OrderCenter.GetOrderReceiptList(ctx, &orderconfig.GetOrderReceiptListReq{
		BrandId: req.GetBrandId(),
		StoreId: req.GetStoreId(),
	})
	if err != nil {
		return &hqOrderReceipt.GetOrderReceiptListResp{
			Code: errorcode.ERR_CODE_NOT_SET,
			Msg:  err.Error(),
		}, nil
	}
	if r.GetCode() != 0 {
		return &hqOrderReceipt.GetOrderReceiptListResp{
			Code: r.GetCode(),
			Msg:  r.GetMsg(),
		}, nil
	}
	data := make([]*hqOrderReceipt.OrderReceipt, 0)
	copier.Copy(&data, r.GetData())

	var physicalPrinterIds []string
	physicalPrinterMap := make(map[string]*hqOrderReceipt.Printer)
	for _, v := range r.GetData() {
		physicalPrinterIds = append(physicalPrinterIds, v.GetPhysicalPrinterId())
	}
	if len(physicalPrinterIds) >= 0 {
		physicalPrinters, _ := o.rpcClient.PrintCenter.QueryPrinter(ctx, &printer.QueryPrinterReq{
			PrinterIds: physicalPrinterIds,
		})
		for _, physicalPrinter := range physicalPrinters {
			physicalPrinterMap[physicalPrinter.GetPrinterId()] = &hqOrderReceipt.Printer{
				Id:          physicalPrinter.GetPrinterId(),
				PrintNodeId: physicalPrinter.GetPrintNodeId(),
				Name:        physicalPrinter.GetName(),
				Type:        physicalPrinter.GetType(),
				PaperSize:   physicalPrinter.GetPaperSize(),
				ConnectType: physicalPrinter.GetConnectType(),
				Address:     physicalPrinter.GetAddress(),
				Encoding:    physicalPrinter.GetEncoding(),
				ErrorCode:   physicalPrinter.GetErrorCode(),
				Direction:   physicalPrinter.GetDirection(),
				Margin:      physicalPrinter.GetMargin(),
				Space:       physicalPrinter.GetSpace(),
				AckLevel:    physicalPrinter.GetAckLevel(),
			}
		}
	}

	for _, p := range data {
		p.Printer = physicalPrinterMap[p.GetPhysicalPrinterId()]
	}

	return &hqOrderReceipt.GetOrderReceiptListResp{
		Code: r.GetCode(),
		Msg:  r.GetMsg(),
		Data: data,
	}, nil
}

func (o *OrderService) DelOrderReceipt(ctx context.Context, req *hqOrderReceipt.DelOrderReceiptReq) (res *hqOrderReceipt.DelOrderReceiptResp, err error) {
	r, err := o.rpcClient.OrderCenter.DelOrderReceipt(ctx, &orderconfig.DelOrderReceiptReq{
		BrandId:   req.GetBrandId(),
		StoreId:   req.GetStoreId(),
		PrinterId: req.GetPrinterId(),
	})
	if err != nil {
		return &hqOrderReceipt.DelOrderReceiptResp{
			Code: errorcode.ERR_CODE_NOT_SET,
			Msg:  err.Error(),
		}, nil
	}
	if r.GetCode() != errorcode.SUCCESS {
		return &hqOrderReceipt.DelOrderReceiptResp{
			Code: r.GetCode(),
			Msg:  r.GetMsg(),
		}, nil
	}

	go func() {
		// 删除真实打印机
		_ = o.rpcClient.PrintCenter.DelPrinter(ctx, &printer.DelPrinterReq{
			PrinterId: r.GetData().GetPhysicalPrinterId(),
		})
		// fixme:  打印配置更新
	}()

	return &hqOrderReceipt.DelOrderReceiptResp{
		Code: r.GetCode(),
		Msg:  r.GetMsg(),
	}, nil

}

func (o *OrderService) UpdateOrderReceipt(ctx context.Context, req *hqOrderReceipt.UpdateOrderReceiptReq) (res *hqOrderReceipt.UpdateOrderReceiptResp, err error) {
	var receiptList []*orderconfig.Receipt
	for _, v := range req.GetReceiptList() {
		receiptList = append(receiptList, &orderconfig.Receipt{
			Receipt: v.GetReceipt(),
			EatType: v.GetEatType(),
			Channel: v.GetChannel(),
			Count:   v.GetCount(),
			Scene:   v.GetScene(),
		})
	}

	r, err := o.rpcClient.OrderCenter.UpdateOrderReceipt(ctx, &orderconfig.UpdateOrderReceiptReq{
		Payload: &orderconfig.OrderReceipt{
			PrinterId:          req.GetPrinterId(),
			BrandId:            req.GetBrandId(),
			StoreId:            req.GetStoreId(),
			Name:               req.GetName(),
			EnableGraphicPrint: req.GetEnableGraphicPrint(),
			EnableBeep:         req.GetEnableBeep(),
			ReceiptList:        receiptList,
			StoreTableAreaNo:   req.GetStoreTableAreaNo(),
			StoreTableAll:      req.GetStoreTableAll(),
		},
		Typ: 1,
	})
	if err != nil {
		return &hqOrderReceipt.UpdateOrderReceiptResp{
			Code: errorcode.ERR_CODE_NOT_SET,
			Msg:  err.Error(),
		}, nil
	}
	if r.GetCode() != 0 {
		return &hqOrderReceipt.UpdateOrderReceiptResp{
			Code: r.GetCode(),
			Msg:  r.GetMsg(),
		}, nil
	}
	data := new(hqOrderReceipt.UpdateOrderReceiptRespData)
	copier.Copy(data, r.GetData())
	return &hqOrderReceipt.UpdateOrderReceiptResp{
		Code: r.GetCode(),
		Msg:  r.GetMsg(),
		Data: data,
	}, nil
}

func (o *OrderService) CountOrderReceipt(ctx context.Context, req *hqOrderReceipt.CountOrderReceiptReq) (res *hqOrderReceipt.CountOrderReceiptResp, err error) {
	r, err := o.rpcClient.OrderCenter.CountOrderReceipt(ctx, &orderconfig.CountOrderReceiptReq{
		BrandId: req.GetBrandId(),
		StoreId: req.GetStoreId(),
	})
	if err != nil {
		return &hqOrderReceipt.CountOrderReceiptResp{
			Code: errorcode.ERR_CODE_NOT_SET,
			Msg:  err.Error(),
		}, nil
	}
	data := make([]*hqOrderReceipt.CountOrderReceiptRespData, 0)
	copier.Copy(&data, r.GetData())
	return &hqOrderReceipt.CountOrderReceiptResp{
		Code: r.GetCode(),
		Msg:  r.GetMsg(),
		Data: data,
	}, nil
}
