package public

import (
	"context"
	"fmt"
	"git.downtown8.com/baseplatform/user-center/hera/kitex_gen/userCenter"
	"git.downtown8.com/rms/g-brand-store/hera/kitex_gen/store"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/internal/message/novu"

	"git.downtown8.com/golang/common/onelog"
	"git.downtown8.com/golang/common/utils"
	"git.downtown8.com/rms/g-common/g-common/constants/errorcode"
	"git.downtown8.com/rms/g-common/g-common/redis"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/internal/config"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/internal/constant/enum"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/internal/pkg"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/internal/rpcclient"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/internal/svc"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/kitex_gen/hqPublicCenter"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/kitex_gen/hqPublicCenter/hqpublicservice"
	"github.com/cloudwego/kitex/server"
	"github.com/sirupsen/logrus"
)

type PublicService struct {
	rpcClient   *rpcclient.Client
	novu        *novu.Client
	redisClient *redis.Client
}

func NewPublicService(client *rpcclient.Client, novu *novu.Client, cfg config.Config) *PublicService {
	rds, err := svc.InitRedis(cfg.Redis)
	if err != nil {
		panic(err)
	}
	return &PublicService{
		rpcClient:   client,
		novu:        novu,
		redisClient: rds,
	}
}

func RegistryPublic(svc *PublicService) func(svr server.Server) error {
	return func(svr server.Server) error {
		return hqpublicservice.RegisterService(svr, svc)
	}
}

// 发送开店任务消息
func (s *PublicService) SendCreateStoreTaskBizMessage(ctx context.Context, req *hqPublicCenter.SendCreateStoreTaskBizMessageReq) (*hqPublicCenter.BaseResp, error) {
	if req.GetBrandId() == 0 || req.GetStoreId() == 0 || req.GetMessageType() == "" {
		return &hqPublicCenter.BaseResp{
			Code: errorcode.PARAMS_ERROR,
			Msg:  errorcode.PARAMS_ERROR_MSG,
		}, nil
	}

	if !pkg.InArray(req.GetMessageType(), []string{enum.MessageTypeOpenPosOrder, enum.MessageTypeConnectPrinter}) {
		return &hqPublicCenter.BaseResp{
			Code: errorcode.PARAMS_ERROR,
			Msg:  errorcode.PARAMS_ERROR_MSG,
		}, nil
	}

	// 查询用户信息
	userInfoResp, err := s.rpcClient.UserCenter.HqUserInfo(ctx, &userCenter.UserCenterHqUserInfoReq{
		BrandId: req.GetBrandId(),
		UserId:  utils.GetUserId(ctx),
	})
	if err != nil {
		return &hqPublicCenter.BaseResp{
			Code: errorcode.SERVER_ERROR,
			Msg:  errorcode.SERVER_ERROR_MSG,
		}, nil
	}
	if userInfoResp.GetCode() != 0 {
		return &hqPublicCenter.BaseResp{
			Code: userInfoResp.GetCode(),
			Msg:  userInfoResp.GetMsg(),
		}, nil
	}

	//// 发送业务消息
	traceId := utils.GetTraceID(ctx)
	// 判断是否POS下单消息
	if req.GetMessageType() == enum.MessageTypeOpenPosOrder {
		// 查询门店点单方式, 判断是否先后付费
		// TODO: 后续需要调用到 品牌门店新服务
		storeOrderTypeResp, err := s.rpcClient.BrandStoreCenter.DetailStore(ctx, &store.DetailStoreRequest{
			BrandId: req.GetBrandId(),
			StoreId: req.GetStoreId(),
		})
		if err != nil {
			return &hqPublicCenter.BaseResp{
				Code: errorcode.SERVER_ERROR,
				Msg:  errorcode.SERVER_ERROR_MSG,
			}, nil
		}
		if storeOrderTypeResp.GetCode() != 0 {
			return &hqPublicCenter.BaseResp{
				Code: storeOrderTypeResp.GetCode(),
				Msg:  storeOrderTypeResp.GetMsg(),
			}, nil
		}

		// 设置默认前付费
		posDinePayModel := enum.PosDinePayModelFront

		// 遍历支付方式配置
		for _, method := range storeOrderTypeResp.GetData().GetStoreOrderMethod() {
			// 判断是否POS堂食后付费
			if method.GetClient() == "pos" &&
				method.GetScene() == "dinein" &&
				method.GetPayModel() == "1" {
				posDinePayModel = enum.PosDinePayModelAfter
				break
			}
		}

		// 发送pos下单新手引导教学
		err = s.novu.PublishPosOrderTutorialWorkflow(ctx, int32(req.GetStoreId()), traceId, &novu.PosOrderTutorialWorkflowPayload{
			BizType:         enum.BizTypeOpenPosOrder,
			ShowBanner:      true,
			EmployeeName:    userInfoResp.GetData().GetUserName(),
			PosDinePayModel: posDinePayModel,
		})
		if err != nil {
			return &hqPublicCenter.BaseResp{
				Code: errorcode.SERVER_ERROR,
				Msg:  errorcode.SERVER_ERROR_MSG,
			}, nil
		}
	} else {
		// 发送pos链接打印机新手引导教学
		err = s.novu.PublishPrinterConnectionTutorialWorkflow(ctx, int32(req.GetStoreId()), traceId, &novu.PrintFailureWorkflowPayload{
			BizType:      enum.BizTypeConnectPrinter,
			ShowBanner:   true,
			EmployeeName: userInfoResp.GetData().GetUserName(),
		})
		if err != nil {
			return &hqPublicCenter.BaseResp{
				Code: errorcode.SERVER_ERROR,
				Msg:  errorcode.SERVER_ERROR_MSG,
			}, nil
		}
	}

	// 将traceId 存入到 redis 集合中, 后续消息已读需要
	bizMessageKey := fmt.Sprintf("%s:%s:%d", enum.NovuCreateStoreTaskBizMessageIdSetKey, req.GetMessageType(), req.GetStoreId())
	onelog.CtxInfoOptions(ctx, "SendCreateStoreTaskBizMessage add traceId to redis", logrus.Fields{
		"bizMessageKey": bizMessageKey,
		"req":           req,
		"traceId":       traceId,
	})

	err = s.redisClient.RedisPool.SAdd(ctx, bizMessageKey, traceId).Err()
	if err != nil {
		onelog.CtxErrorOptions(ctx, "SendCreateStoreTaskBizMessage add traceId to redis failed", logrus.Fields{
			"err": err,
		})
	}

	return &hqPublicCenter.BaseResp{
		Code: errorcode.SUCCESS,
		Msg:  errorcode.SUCCESS_MSG,
	}, nil
}
