package amqp

import (
	"context"
	"fmt"
	"git.downtown8.com/rms/orchestrator/g-hq-orchestrator/internal/config"
	"github.com/google/wire"
	"go.opentelemetry.io/otel"
	"go.opentelemetry.io/otel/codes"
	"go.opentelemetry.io/otel/propagation"
	"log"
	"net/url"
	"sync"
	"time"

	amqp "github.com/rabbitmq/amqp091-go"
)

type RabbitMQ struct {
	channels            []*channel    // 发送消息的通道
	declarationChannel  *amqp.Channel // 初始化通道 不发送业务消息 只做申明等操作
	connection          *amqp.Connection
	declareExchangeFunc func(exchange string) error
	mu                  sync.RWMutex
	cfg                 config.Config
	isClosed            bool
	waitConn            chan struct{}
}
type channel struct {
	ch    *amqp.Channel
	inUse bool
	mu    sync.RWMutex
}

func buildRabbitMQURL(username, password, hostname string, port int, vhost string) string {
	u := &url.URL{
		Scheme: "amqp",
		User:   url.UserPassword(username, password),
		Host:   fmt.Sprintf("%s:%d", hostname, port),
		Path:   fmt.Sprintf("/%s", vhost),
	}
	return u.String()
}

var ProviderSet = wire.NewSet(NewRabbitMQ)

func NewRabbitMQ(cfg config.Config) (*RabbitMQ, error) {
	dsn := buildRabbitMQURL(cfg.RabbitMq.UserName, cfg.RabbitMq.Password, cfg.RabbitMq.Host, cfg.RabbitMq.Port, cfg.RabbitMq.Vhost)
	connection, err := amqp.Dial(dsn)
	if err != nil {
		return nil, err
	}
	rabbitMQ := &RabbitMQ{
		cfg:        cfg,
		connection: connection,
		channels:   []*channel{},
		waitConn:   make(chan struct{}, 1),
	}
	rabbitMQ.buildChannel()

	rabbitMQ.declareExchange()
	rabbitMQ.notifyReturnClose()

	return rabbitMQ, nil
}

const maxBackoff = 16

func (r *RabbitMQ) reConnect() {
	backoff := 1
	for {
		dsn := buildRabbitMQURL(r.cfg.RabbitMq.UserName, r.cfg.RabbitMq.Password, r.cfg.RabbitMq.Host, r.cfg.RabbitMq.Port, r.cfg.RabbitMq.Vhost)
		connection, err := amqp.Dial(dsn)
		if err != nil {
			time.Sleep(time.Duration(backoff) * 500 * time.Millisecond)
			if backoff < maxBackoff {
				backoff <<= 1
			}
			continue
		}
		r.mu.Lock()
		if r.connection != nil {
			r.connection.Close() // 关闭旧连接
		}
		r.connection = connection
		r.buildChannel()
		r.notifyReturnClose()
		r.mu.Unlock()
		return
	}
}

func (r *RabbitMQ) buildChannel() {
	r.channels = make([]*channel, 0)
	ch, err := r.connection.Channel()
	if err != nil {
		fmt.Println(err)
	}

	r.declarationChannel = ch

	for i := 0; i < r.cfg.RabbitMq.ChannelNum; i++ {
		ch, err = r.connection.Channel()
		if err != nil {
			fmt.Println(err)
			continue
		}
		r.channels = append(r.channels, &channel{
			ch:    ch,
			inUse: false,
			mu:    sync.RWMutex{},
		})
	}
}

func (r *RabbitMQ) getChannel() *channel {
	r.mu.RLock()
	if r.isClosed {
		select {
		case <-r.waitConn:
			break
		default:
		}

	}
	r.mu.RUnlock()

	for {
		r.mu.RLock()
		for _, ch := range r.channels {
			ch.mu.Lock()
			if ch.inUse { // 如果通道使用
				ch.mu.Unlock()
				continue
			}
			ch.inUse = true
			ch.mu.Unlock()
			r.mu.RUnlock()
			return ch
		}
		r.mu.RUnlock()
		time.Sleep(1 * time.Millisecond)
	}
}

func (r *RabbitMQ) notifyReturnClose() {
	ch := make(chan *amqp.Error)
	r.connection.NotifyClose(ch)

	go func() {
		for _ = range ch { // 如果
			r.reConnect()
			return
		}
	}()
}

const (
	exchangeTypeTopic   = "topic"
	exchangeTypeDelayed = "x-delayed-message"
)

func (r *RabbitMQ) declareExchange() {
	exchangeMap := make(map[string]struct{})
	r.declareExchangeFunc = func(exchange string) error {
		r.mu.Lock()
		defer r.mu.Unlock()
		if _, ok := exchangeMap[exchange]; ok {
			return nil
		}
		exchangeMap[exchange] = struct{}{}
		return r.declarationChannel.ExchangeDeclare(exchange, // 交换机名称
			exchangeTypeTopic, // 交换机类型
			true,              // 是否持久化
			false,             // 是否自动删除
			false,             // 是否为内部交换机
			false,             // 是否阻塞
			nil,               // 其他属性
		)
	}
}

func (r *RabbitMQ) Publish(ctx context.Context, exchange, key string, data []byte) error {
	err := r.declareExchangeFunc(exchange)
	if err != nil {
		return err
	}
	ch := r.getChannel()

	defer func() {
		ch.mu.Lock()
		ch.inUse = false
		ch.mu.Unlock()
	}()

	tracer := otel.Tracer("rabbitmq")
	ctx, span := tracer.Start(ctx, "rabbitmq.publish")
	defer span.End()

	headers := make(amqp.Table)

	// 创建 string map 用于传播
	carrier := make(map[string]string)
	propagator := otel.GetTextMapPropagator()
	propagator.Inject(ctx, propagation.MapCarrier(carrier))

	// 将 string map 转换为 amqp.Table 类型
	for k, v := range carrier {
		headers[k] = v // RabbitMQ 要求值必须是特定类型，这里隐式转换为 interface{}
	}

	err = ch.ch.PublishWithContext(
		ctx,
		exchange,
		key,
		false,
		false,
		amqp.Publishing{
			ContentType:  "text/plain",
			DeliveryMode: amqp.Persistent,
			Body:         data,
			Headers:      headers,
		})

	if err != nil {
		span.SetStatus(codes.Error, "消息发布失败") // 标记状态
		span.RecordError(err)                       // 记录错误详情
	}
	
	return err
}

type Msg struct {
	Exchange   string
	RoutingKey string
	Data       []byte
}

func (r *RabbitMQ) PublishMsg(ctx context.Context, msg Msg) {
	go func() {
		err := r.Publish(ctx, msg.Exchange, msg.RoutingKey, msg.Data)
		if err != nil {
			// fixme
		}
	}()

	return
}

func (r *RabbitMQ) BindQueue(queueName string, autoAck bool) (<-chan amqp.Delivery, error) {
	ch := r.getChannel()
	// 声明要消费的队列
	q, err := ch.ch.QueueDeclare(
		queueName, // 队列名称
		true,      // 是否持久化
		false,     // 是否自动删除
		false,     // 是否排他
		false,     // 是否等待服务器响应
		nil,       // 额外的属性
	)
	if err != nil {
		log.Fatalf("Failed to declare a queue: %s", err)
	}
	msg, err := ch.ch.Consume(q.Name, // 队列名称
		"",      // 消费者名称
		autoAck, // 是否自动确认
		false,   // 是否排他
		false,   // 是否等待服务器响应
		false,   // 无本地队列
		nil,     // 额外的属性
	)
	if err != nil {
		log.Fatalf("Failed to register a queue: %s", err)
	}
	return msg, nil
}
